
{******************************************}
{                                          }
{             FastScript v1.9              }
{  IB Objects classes and functions        }
{                                          }
{ Created by: Andreas Hesse                }
{ E-mail: delphi@ah3000.de                 }
{                                          }
{******************************************}


// This unit is based upon unit 'fs_idbrtti.pas'.
//
// TField -> TIB_Column
// TDataSet -> TIB_Dataset
// TDataSource -> TIB_DataSource
// TFields -> TIB_Row

// TLocateOptions -> TIB_LocateOptions
// TFilterOption -> TIB_FilterOption

unit fs_iib_rtti;

interface

{$i fs.inc}

uses
  SysUtils, Classes, fs_iinterpreter, fs_itools, fs_iclassesrtti, fs_ievents,
  IB_Session, IB_Components;


type
  TfsIB_RTTI = class(TComponent); // fake component

  TfsIB_DatasetNotifyEvent = class(TfsCustomEvent)
  public
    procedure DoEvent(Dataset: TIB_Dataset);
    function GetMethod: Pointer; override;
  end;

  TfsIB_FilterRecordEvent = class(TfsCustomEvent)
  public
    procedure DoEvent(ARow: TIB_Row; var Accept: Boolean);
    function GetMethod: Pointer; override;
  end;

  TfsIB_ColumnGetTextEvent = class(TfsCustomEvent)
  public
    procedure DoEvent(Sender: TIB_Column; var Text: String);
    function GetMethod: Pointer; override;
  end;


implementation

type
  TFunctions = class(TfsRTTIModule)
  private
    function CallMethod(Instance: TObject; ClassType: TClass; const MethodName: String; Caller: TfsMethodHelper): Variant;
    function GetProp(Instance: TObject; ClassType: TClass; const PropName: String): Variant;
    procedure SetProp(Instance: TObject; ClassType: TClass; const PropName: String; Value: Variant);
  public
    constructor Create(AScript: TfsScript); override;
  end;


{ TfsIB_DatasetNotifyEvent }

procedure TfsIB_DatasetNotifyEvent.DoEvent(Dataset: TIB_Dataset);
begin
  CallHandler([Dataset]);
end;

function TfsIB_DatasetNotifyEvent.GetMethod: Pointer;
begin
  Result := @TfsIB_DatasetNotifyEvent.DoEvent;
end;


{ TfsIB_FilterRecordEvent }

procedure TfsIB_FilterRecordEvent.DoEvent(ARow: TIB_Row; var Accept: Boolean);
begin
  CallHandler([ARow, Accept]);
  Accept := Handler.Params[1].Value;
end;

function TfsIB_FilterRecordEvent.GetMethod: Pointer;
begin
  Result := @TfsIB_FilterRecordEvent.DoEvent;
end;


{ TfsIB_ColumnGetTextEvent }

procedure TfsIB_ColumnGetTextEvent.DoEvent(Sender: TIB_Column; var Text: String);
begin
  CallHandler([Sender, Text]);
  Text := Handler.Params[1].Value;
end;

function TfsIB_ColumnGetTextEvent.GetMethod: Pointer;
begin
  Result := @TfsIB_ColumnGetTextEvent.DoEvent;
end;


{ TFunctions }

constructor TFunctions.Create(AScript: TfsScript);
begin
  inherited Create(AScript);
  with AScript do
  begin
    AddEnumSet('TIB_LocateOption', 'lopCaseInsensitive, lopPartialKey, lopFindNearest');
    AddEnumSet('TIB_FilterOption', 'fopCaseInsensitive, fopNoPartialCompare');

    with AddClass(TIB_Column, 'TComponent') do
    begin
      AddProperty('AsBoolean', 'Boolean', GetProp, SetProp);
      AddProperty('AsCurrency', 'Currency', GetProp, SetProp);
      AddProperty('AsDateTime', 'TDateTime', GetProp, SetProp);
      AddProperty('AsFloat', 'Double', GetProp, SetProp);
      AddProperty('AsInteger', 'Integer', GetProp, SetProp);
      AddProperty('AsString', 'String', GetProp, SetProp);
      AddProperty('AsVariant', 'Variant', GetProp, SetProp);
      AddProperty('DisplayName', 'String', GetProp, nil);
      AddProperty('DisplayText', 'String', GetProp, nil);
      AddProperty('IsNull', 'Boolean', GetProp, nil);
      AddProperty('Size', 'Integer', GetProp, nil);
      AddProperty('Value', 'Variant', GetProp, SetProp);
      AddEvent('OnGetText', TfsIB_ColumnGetTextEvent);
    end;

    with AddClass(TIB_Row, 'TObject') do
      AddDefaultProperty('Columns', 'Integer', 'TIB_Column', CallMethod, True);

    AddClass(TIB_ColumnNumBase, 'TIB_Column');
    AddClass(TIB_ColumnNumeric, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnFloat, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnDouble, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnBoolean, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnSmallInt, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnInteger, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnInt64, 'TIB_ColumnNumBase');
    AddClass(TIB_ColumnQuad, 'TIB_ColumnNumBase');

    AddClass(TIB_ColumnDateTime, 'TIB_Column');
    AddClass(TIB_ColumnDate, 'TIB_ColumnDateTime');
    AddClass(TIB_ColumnTime, 'TIB_ColumnDateTime');
    AddClass(TIB_ColumnTimeStamp, 'TIB_ColumnDateTime');

    AddClass(TIB_ColumnText, 'TIB_Column');
    AddClass(TIB_ColumnVarText, 'TIB_ColumnText');

    AddClass(TIB_ColumnDB_KEY, 'TIB_Column');

    with AddClass(TIB_ColumnBlob, 'TIB_Column') do
    begin
      AddMethod('procedure LoadFromFile(const FileName: String)', CallMethod);
      //AddMethod('procedure LoadFromStream(Stream: TStream)', CallMethod);
      AddMethod('procedure SaveToFile(const FileName: String)', CallMethod);
      //AddMethod('procedure SaveToStream(Stream: TStream)', CallMethod);
    end;
    AddClass(TIB_ColumnBinary, 'TIB_ColumnBlob');
    AddClass(TIB_ColumnMemo, 'TIB_ColumnBlob');
    AddClass(TIB_ColumnBLR, 'TIB_ColumnBlob');

    AddClass(TIB_ColumnArray, 'TIB_Column');

    AddClass(TIB_DataSource, 'TComponent');

    with AddClass(TIB_DataSet, 'TComponent') do
    begin
      AddMethod('procedure Open', CallMethod);
      AddMethod('procedure Close', CallMethod);
      AddMethod('procedure First', CallMethod);
      AddMethod('procedure Last', CallMethod);
      AddMethod('procedure Next', CallMethod);
      AddMethod('procedure Prior', CallMethod);
      AddMethod('procedure Cancel', CallMethod);
      AddMethod('procedure Delete', CallMethod);
      AddMethod('procedure Post', CallMethod);
      AddMethod('procedure Append', CallMethod);
      AddMethod('procedure Insert', CallMethod);
      AddMethod('procedure Edit', CallMethod);

      AddMethod('function FieldByName(const FieldName: string): TField', CallMethod);
      AddMethod('procedure GetFieldNames(List: TStrings)', CallMethod);
      AddMethod('function IsEmpty: Boolean', CallMethod);
      AddMethod('procedure EnableControls', CallMethod);
      AddMethod('procedure DisableControls', CallMethod);

      AddProperty('Bof', 'Boolean', GetProp, nil);
      AddProperty('Eof', 'Boolean', GetProp, nil);
      AddProperty('FieldCount', 'Integer', GetProp, nil);
      AddProperty('Fields', 'TIB_Row', GetProp, nil);
      AddProperty('Active', 'Boolean', GetProp, SetProp);

      AddEvent('BeforeOpen', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterOpen', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforeClose', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterClose', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforeInsert', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterInsert', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforeEdit', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterEdit', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforePost', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterPost', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforeCancel', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterCancel', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforeDelete', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterDelete', TfsIB_DatasetNotifyEvent);
      AddEvent('BeforeScroll', TfsIB_DatasetNotifyEvent);
      AddEvent('AfterScroll', TfsIB_DatasetNotifyEvent);
      AddEvent('OnCalcFields', TfsIB_DatasetNotifyEvent);
      AddEvent('OnFilterRecord', TfsIB_FilterRecordEvent);
      AddEvent('OnNewRecord', TfsIB_DatasetNotifyEvent);
    end;

    with AddClass(TIB_BDataset, 'TIB_DataSet') do
    begin
      AddMethod('function FindFirst: Boolean', CallMethod);
      AddMethod('function FindLast: Boolean', CallMethod);
      AddMethod('function FindNext: Boolean', CallMethod);
      AddMethod('function FindPrior: Boolean', CallMethod);
      AddMethod('function Locate(const KeyFields: string; const KeyValues: Variant; Options: TIB_LocateOptions): Boolean', CallMethod);
      AddProperty('Filter', 'string', GetProp, SetProp);
      AddProperty('Filtered', 'Boolean', GetProp, SetProp);
      AddProperty('FilterOptions', 'TFilterOptions', GetProp, SetProp);
    end;

  end;
end;

function TFunctions.CallMethod(Instance: TObject; ClassType: TClass; const MethodName: String; Caller: TfsMethodHelper): Variant;
var
  _TDataSet: TIB_DataSet;
  _TBDataSet: TIB_BDataset;

  function IntToLocateOptions(i: Integer): TIB_LocateOptions;
  begin
    Result := [];
    if (i and 1) <> 0 then
      Result := Result + [lopCaseInsensitive];
    if (i and 2) <> 0 then
      Result := Result + [lopPartialKey];
  end;

begin
  Result := 0;

  if ClassType = TIB_Row then
  begin
    if MethodName = 'IB_ROW.GET' then
      Result := Integer(TIB_Row(Instance)[Caller.Params[0]])
  end
  else if ClassType = TIB_ColumnBlob then
  begin
    if MethodName = 'LOADFROMFILE' then
      TIB_ColumnBlob(Instance).LoadFromFile(Caller.Params[0])
    else if MethodName = 'SAVETOFILE' then
      TIB_ColumnBlob(Instance).SaveToFile(Caller.Params[0])
  end
  else if ClassType = TIB_DataSet then
  begin
    _TDataSet := TIB_DataSet(Instance);
    if MethodName = 'OPEN' then
      _TDataSet.Open
    else if MethodName = 'CLOSE' then
      _TDataSet.Close
    else if MethodName = 'FIRST' then
      _TDataSet.First
    else if MethodName = 'LAST' then
      _TDataSet.Last
    else if MethodName = 'NEXT' then
      _TDataSet.Next
    else if MethodName = 'PRIOR' then
      _TDataSet.Prior
    else if MethodName = 'CANCEL' then
      _TDataSet.Cancel
    else if MethodName = 'DELETE' then
      _TDataSet.Delete
    else if MethodName = 'POST' then
      _TDataSet.Post
    else if MethodName = 'APPEND' then
      _TDataSet.Append
    else if MethodName = 'INSERT' then
      _TDataSet.Insert
    else if MethodName = 'EDIT' then
      _TDataSet.Edit
    else if MethodName = 'FIELDBYNAME' then
      Result := Integer(_TDataSet.FieldByName(Caller.Params[0]))
    else if MethodName = 'GETFIELDNAMES' then
      _TDataSet.GetFieldNamesList(TStrings(Integer(Caller.Params[0])))
    else if MethodName = 'ENABLECONTROLS' then
      _TDataSet.EnableControls
    else if MethodName = 'DISABLECONTROLS' then
      _TDataSet.DisableControls
  end
  else if ClassType = TIB_BDataSet then
  begin
    _TBDataSet := TIB_BDataSet(Instance);
    if MethodName = 'FINDFIRST' then
      Result := _TBDataSet.FindFirst
    else if MethodName = 'FINDLAST' then
      Result := _TBDataSet.FindLast
    else if MethodName = 'FINDNEXT' then
      Result := _TBDataSet.FindNext
    else if MethodName = 'FINDPRIOR' then
      Result := _TBDataSet.FindPrior
    else if MethodName = 'ISEMPTY' then
      Result := _TBDataSet.IsEmpty
    else if MethodName = 'LOCATE' then
      Result := _TBDataSet.Locate(Caller.Params[0], Caller.Params[1], IntToLocateOptions(Caller.Params[2]))
  end


end;

function TFunctions.GetProp(Instance: TObject; ClassType: TClass; const PropName: String): Variant;
var
  _TField: TIB_Column;
  _TDataSet: TIB_DataSet;
  _TBDataSet: TIB_BDataset;

  function FilterOptionsToInt(f: TIB_FilterOptions): Integer;
  begin
    Result := 0;
    if fopCaseInsensitive in f then
      Result := Result or 1;
    if fopNoPartialCompare in f then
      Result := Result or 2;
  end;

begin
  Result := 0;

  if ClassType = TIB_Column then
  begin
    _TField := TIB_Column(Instance);
    if PropName = 'ASBOOLEAN' then
      Result := _TField.AsBoolean
    else if PropName = 'ASCURRENCY' then
      Result := _TField.AsCurrency
    else if PropName = 'ASDATETIME' then
      Result := _TField.AsDateTime
    else if PropName = 'ASFLOAT' then
      Result := _TField.AsFloat
    else if PropName = 'ASINTEGER' then
      Result := _TField.AsInteger
    else if PropName = 'ASSTRING' then
      Result := _TField.AsString
    else if PropName = 'ASVARIANT' then
      Result := _TField.AsVariant
    else if PropName = 'DATATYPE' then
      //Result := _TField.DataType
    else if PropName = 'DISPLAYNAME' then
      Result := _TField.DisplayName
    else if PropName = 'DISPLAYTEXT' then
      Result := _TField.DisplayText
    else if PropName = 'ISNULL' then
      Result := _TField.IsNull
    else if PropName = 'SIZE' then
      //Result := _TField.Size
    else if PropName = 'VALUE' then
      Result := _TField.Value
  end
  else if ClassType = TIB_DataSet then
  begin
    _TDataSet := TIB_DataSet(Instance);
    if PropName = 'BOF' then
      Result := _TDataSet.Bof
    else if PropName = 'EOF' then
      Result := _TDataSet.Eof
    else if PropName = 'FIELDCOUNT' then
      Result := _TDataSet.FieldCount
    else if PropName = 'FIELDS' then
      Result := Integer(_TDataSet.Fields)
    else if PropName = 'ACTIVE' then
      Result := _TDataSet.Active
  end
  else if ClassType = TIB_DataSet then
  begin
    _TBDataSet := TIB_BDataSet(Instance);
    if PropName = 'FILTER' then
      Result := _TBDataSet.Filter
    else if PropName = 'FILTERED' then
      Result := _TBDataSet.Filtered
    else if PropName = 'FILTEROPTIONS' then
      Result := FilterOptionsToInt(_TBDataSet.FilterOptions)
  end
end;

procedure TFunctions.SetProp(Instance: TObject; ClassType: TClass; const PropName: String; Value: Variant);
var
  _TField: TIB_Column;
  _TDataSet: TIB_DataSet;
  _TBDataSet: TIB_BDataset;

  function IntToFilterOptions(i: Integer): TIB_FilterOptions;
  begin
    Result := [];
    if (i and 1) <> 0 then
      Result := Result + [fopCaseInsensitive];
    if (i and 2) <> 0 then
      Result := Result + [fopNoPartialCompare];
  end;

begin
  if ClassType = TIB_Column then
  begin
    _TField := TIB_Column(Instance);
    if PropName = 'ASBOOLEAN' then
      _TField.AsBoolean := Value
    else if PropName = 'ASCURRENCY' then
      _TField.AsCurrency := Value
    else if PropName = 'ASDATETIME' then
      _TField.AsDateTime := Value
    else if PropName = 'ASFLOAT' then
      _TField.AsFloat := Value
    else if PropName = 'ASINTEGER' then
      _TField.AsInteger := Value
    else if PropName = 'ASSTRING' then
      _TField.AsString := Value
    else if PropName = 'ASVARIANT' then
      _TField.AsVariant := Value
    else if PropName = 'VALUE' then
      _TField.Value := Value
  end
  else if ClassType = TIB_DataSet then
  begin
    _TDataSet := TIB_DataSet(Instance);
    if PropName = 'ACTIVE' then
      _TDataSet.Active := Value
  end
  else if ClassType = TIB_BDataSet then
  begin
    _TBDataSet := TIB_BDataSet(Instance);
    if PropName = 'FILTER' then
      _TBDataSet.Filter := Value
    else if PropName = 'FILTERED' then
      _TBDataSet.Filtered := Value
    else if PropName = 'FILTEROPTIONS' then
      _TBDataSet.FilterOptions := IntToFilterOptions(Value)
  end
end;


initialization
  fsRTTIModules.Add(TFunctions);

finalization
  if fsRTTIModules <> nil then
    fsRTTIModules.Remove(TFunctions);

end.
